package ru.landgrafhomyak.db.skeleton1.api.errors

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.ReflectionApi
import ru.landgrafhomyak.db.skeleton1.api.statement.InputParam
import ru.landgrafhomyak.db.skeleton1.api.statement._Statement

@ReflectionApi
@OptIn(ExperimentalCoroutinesApi::class)
public class ScopeAlreadyFinishedException : IllegalStateException, CopyableThrowable<ScopeAlreadyFinishedException> {
	public val scope: Any
	public val member: Any

	public constructor(scope: Any, member: Any) : super() {
		this.scope = scope
		this.member = member
	}

	public constructor(scope: Any, member: Any, customMessage: String) : super(customMessage) {
		this.scope = scope
		this.member = member
	}

	public constructor(scope: Any, member: Any, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.scope = scope
		this.member = member
	}

	public constructor(scope: Any, member: Any, cause: Throwable) : super(cause) {
		this.scope = scope
		this.member = member
	}

	public constructor(
		scope: Any, member: Any,
		message: String?,
		cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause) {
		this.scope = scope
		this.member = member
	}


	override fun createCopy(): ScopeAlreadyFinishedException? {
		return ScopeAlreadyFinishedException(
			scope = this.scope,
			member = this.member,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(scope: Any, member: Any): String =
			"Attempt to access $member of $scope after it was closed"
	}
}