package ru.landgrafhomyak.db.skeleton1.api.module

import kotlin.jvm.JvmName
import ru.landgrafhomyak.db.skeleton1.api.ReflectionApi
import ru.landgrafhomyak.db.skeleton1.api.LowLevelApi
import ru.landgrafhomyak.db.skeleton1.api.statement.QueriesCompiler
import ru.landgrafhomyak.db.skeleton1.api.runtime.Transaction
import ru.landgrafhomyak.db.skeleton1.api.table.Table


public interface Module<mUE : Any> {
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("uExt")
	public val uExt: mUE

	@ReflectionApi
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("getVersionKey")
	public val versionKey: String

	@ReflectionApi
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("getParentModule")
	public val parentModule: Module<*>?

	@ReflectionApi
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("getPathFromParentModuleRoot")
	public val pathFromParentModuleRoot: List<String>?

	@ReflectionApi
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("getMemberTables")
	public val memberTables: Collection<Table<*, mUE>>

	@ReflectionApi
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("getMemberObjects")
	public val memberObjects: Collection<ModuleScopedObject<mUE>>

	@ReflectionApi
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("getMemberSubmodules")
	public val memberSubmodules: Collection<Module<*>>

	public interface Constructor<mUE : Any> {
		@Suppress("INAPPLICABLE_JVM_NAME")
		@get:JvmName("versionKey")
		public val versionKey: String

		public fun createSchema(context: ModuleConstructorScope<mUE>): mUE

		public suspend fun initData(ext: mUE, compiler: QueriesCompiler, transaction: Transaction) {}
	}

	public interface Upgrade<nmUE : Any, omUE : Any> {
		@Suppress("INAPPLICABLE_JVM_NAME")
		@get:JvmName("versionKey")
		public val versionKey: String

		public fun upgradeSchema(context: ModuleUpgradeScope<nmUE, omUE>): nmUE

		public suspend fun upgradeData(ext: nmUE, compiler: QueriesCompiler, transaction: Transaction) {}
	}

	public interface Template<@Suppress("unused") mUE : Any> {
		@LowLevelApi
		public fun getConstructor(): Constructor<Any>

		@LowLevelApi
		public fun getUpgrade(version: String): Upgrade<Any, Any>?
	}
}