package ru.landgrafhomyak.db.skeleton1.api.module

import ru.landgrafhomyak.db.skeleton1.api.LowLevelApi
import ru.landgrafhomyak.db.skeleton1.api.statement.RawStatement
import ru.landgrafhomyak.db.skeleton1.api.statement._Statement
import ru.landgrafhomyak.db.skeleton1.api.runtime.InputRow
import ru.landgrafhomyak.db.skeleton1.api.table.Table


public interface ModuleUpgradeScope<nmUE : Any, omUE : Any> : ModuleConstructorScope<nmUE> {
	// Table -------------------------------------------------------------------
	public fun <tUE : Any> keepTable(table: Table<tUE, omUE>): Table<tUE, nmUE>
	public fun <tUE : Any> keepAndMoveTable(
		table: Table<tUE, omUE>,
		newLocation: NameNode<nmUE>
	): Table<tUE, nmUE>

	public fun <ntUE : Any, otUE : Any> upgradeTable(
		oldTable: Table<otUE, omUE>,
		upgrade: Table.Upgrade<ntUE, otUE>
	): Table<ntUE, nmUE>

	public fun <ntUE : Any, otUE : Any> upgradeAndMoveTable(
		table: Table<otUE, omUE>,
		newLocation: NameNode<nmUE>,
		upgrade: Table.Upgrade<ntUE, otUE>
	): Table<ntUE, nmUE>

	public fun deleteTable(table: Table<*, omUE>)
	public fun <tUE : Any> deleteTableAfterModuleUpgrade(table: Table<tUE, omUE>): Table<tUE, nmUE>

	public fun exportTable(table: Table<*, omUE>)
	public fun <tUE : Any> importTable(
		table: Table<tUE, *>,
		location: NameNode<nmUE>
	): Table<tUE, nmUE>

	public fun <ntUE : Any, otUE : Any> importAndUpgradeTable(
		table: Table<otUE, *>,
		location: NameNode<nmUE>,
		upgrade: Table.Upgrade<ntUE, otUE>
	): Table<ntUE, nmUE>

	// Object ------------------------------------------------------------------

	public fun keepModuleScopedObject(obj: ModuleScopedObject<omUE>): ModuleScopedObject<nmUE>
	public fun keepAndMoveModuleScopedObject(
		obj: ModuleScopedObject<omUE>,
		newLocation: NameNode<nmUE>,
	): ModuleScopedObject<nmUE>

	public fun deleteModuleScopedObject(obj: ModuleScopedObject<omUE>)
	public fun deleteModuleScopedObjectAfterModuleUpgrade(obj: ModuleScopedObject<omUE>): ModuleScopedObject<nmUE>

	public fun exportModuleScopedObject(obj: ModuleScopedObject<omUE>)
	public fun importModuleScopedObject(
		obj: ModuleScopedObject<*>,
		location: NameNode<nmUE>
	): ModuleScopedObject<nmUE>


	// Module ------------------------------------------------------------------

	public fun <smUE : Any> keepModule(module: Module<smUE>): Module<smUE>
	public fun <smUE : Any> keepAndMoveModule(oldModule: Module<smUE>, newNs: NameNode<nmUE>): Module<smUE>

	public fun <smUE : Any> upgradeModule(oldModule: Module<*>, template: Module.Template<smUE>): Module<smUE>
	public fun <smUE : Any> upgradeAndMoveModule(
		oldModule: Module<*>,
		newNamespace: NameNode<nmUE>,
		template: Module.Template<smUE>
	): Module<smUE>

	public fun deleteModule(oldModule: Module<*>)
	public fun <smUE : Any> deleteModuleAfterUpgrade(oldModule: Module<smUE>): Module<smUE>

	public fun exportModule(table: Module<*>)

	public fun <smUE : Any> importModule(
		module: Module<smUE>,
		location: NameNode<nmUE>,
	): Module<smUE>

	public fun <nsmUE : Any, osmUE : Any> importAndUpgradeModule(
		module: Module<osmUE>,
		location: NameNode<nmUE>,
		upgrade: Module.Upgrade<nsmUE, osmUE>
	): Module<nsmUE>

	// Statements --------------------------------------------------------------

	public fun addStatementAfterUpgrade(stmt: _Statement.Void2Void<*>)
	public fun addStatementAfterUpgrade(stmt: RawStatement.Void2Void.ImplementationsProvider<*>)

	@LowLevelApi
	public fun <sUE : Any> _addStatementAfterUpgrade(stmt: _Statement.Params2Void<sUE>): InputRow._Scope<sUE, Unit>

	@LowLevelApi
	public fun <sUE : Any> _addStatementAfterUpgrade(stmt: RawStatement.Params2Void.ImplementationsProvider<sUE>): InputRow._Scope<sUE, Unit>
}